# ----------------------------------------------------------------------
# This file was generated by /Users/humberto.a.sanchez.ii/PycharmProjects/PyUt/venv-pyenv-3.9.0/bin/img2py
#
from wx.lib.embeddedimage import PyEmbeddedImage

embeddedImage: PyEmbeddedImage = PyEmbeddedImage(
    b'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAK3RFWHRDcmVhdGlvbiBUaW1l'
    b'AEZyIDE1IE5vdiAyMDAyIDEwOjA4OjI5ICswMTAw9aJixgAAAAd0SU1FB9MDBgs0KlZxNO0A'
    b'AAAJcEhZcwAACxIAAAsSAdLdfvwAAAAEZ0FNQQAAsY8L/GEFAAACuUlEQVR42qWTXUhTYRjH'
    b'/2dnm57NzQ+UNVOMyj5ofVCRdVGQRhCY+UGWFnhlSBB0320hdBV1EZE3ERFCV0oqkaWBWUim'
    b'lTE3pvm1jmdnbmfz7Jyz89W7BYMSr3xenvfifd7nx/P+3+cBtmjU/wftM7vKRA43XfnF9XkW'
    b'525F10w2GQ0sCmyfXGU+XqwRYpsCuiKHW5wRT8/JytqiwnwX+PQC4hIPUUphOcZjdP4nP+fi'
    b'OhK12sAGQCd38HKV7Ht5vrKJDhnTCCSm8PTtZzgsDpw7sA2MaYepGRiYC6p+KtKIBgzkAK0z'
    b'+7wew+u/4ut0B7RxsMoSHrx+h3BzHLSVwpHBahzzOsHADt0w8Hz6W0Rk1L1oQ8ySAayFk7dO'
    b'ba9zL8jfsSLOZ5OXmqPZ5IxNXQjiV4JGTE1CVtM4WuwtwyK6MrEswJFX1FDKlGBGmMDDwRGw'
    b'LQLsVit004BJPGPDZ75Ao8rBqwk4GTug42IOwFic1cvJOTwb+QruahIUDVz71ABeYSGbChon'
    b'TmchvceHsd97FpKmII+i9+QAkq5hMuzHpYrrGOM+omO8DaNr/SiwuQmcwYo8hdvBVoytv8EJ'
    b'Tw1UE0TQv+JbM1tY4AJ1O3y+D+oQukP9qHSVoBpeUn6WjyJbIcKrU7jL3oCqGohFRKiSHsgB'
    b'Znm2bynG+crtbhg2Cyw6YJAvy6xsmTQNRSaCmjSEtTSSKRWmir7cE5Ri7dGr0KSgk/spRcG6'
    b'JJHLFNzWgiyAJqLImkoS0xDiClhOjEDDk2wss+m9WI+3i8FoRG6pKHRZRKKJTlrEr/sxqbxH'
    b'KDILnpcQJR7+LaqKTWvFPfzYOAsv0FSQsPUc8npKHPlWpNIyaeM0FIlCMqFglZM4jdE7cAdD'
    b'mw4T7qMUImkSE/WMnd6paxTSKS2INHkzQ8ruRnyrE/yP/QEVnEpqw3XnNAAAAABJRU5ErkJg'
    b'gg==')
