# ----------------------------------------------------------------------
# This file was generated by /Users/humberto.a.sanchez.ii/PycharmProjects/PyUt/venv-pyenv-3.8.5-alternate/bin/img2py
#
from wx.lib.embeddedimage import PyEmbeddedImage

embeddedImage: PyEmbeddedImage = PyEmbeddedImage(
    b'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAK3RFWHRDcmVhdGlvbiBUaW1l'
    b'AEZyIDE1IE5vdiAyMDAyIDEwOjA4OjI5ICswMTAw9aJixgAAAAd0SU1FB9QHBwgcNzpVudoA'
    b'AAAJcEhZcwAACvAAAArwAUKsNJgAAAAEZ0FNQQAAsY8L/GEFAAADC0lEQVR42k2T3W9TdRjH'
    b'P+ecnq7bWgp0ZejCeIkQdUCCgBoTL4yYiBlGloB3mngFFyI3Zjf8AcCFSggwL0ggMVEv8AV1'
    b'QUkgIwhcsOCFA0Y3cO26denO1m6lOz1vP5/TtYSTfE9+L8/z/J6X71fjuW90PfFWn8O0a73m'
    b'ytRLnh8wX5wbm7H58+t5Lgw6FMXME6imj9Zc2Ft43053XGjb19dpbtwEU1ko5mFxnqUnWTIj'
    b'+cKNeffLz0v8JuYVgR/6GeHP28zeWk/P5RVH+lcYdhn39jWMUgHmiiiriGkV6FwdjSdttzfm'
    b'MHXL5Un4ZpiJUewi4XSvuZ78oj/hDA0yNh1g620kXUve8AhmS/inrmAUc3TMZgzjKa9fqXL3'
    b'KcxIAEdf0Dma6O1L898oQW6caN+nRI9fxH6cJchOoL7/B3P7m6j8eL307gTpD1vYL5tuQUTX'
    b'E7F95touuD9MrCNF+88D6LE2vGPn8b69RSS1Fr45gpZ5UK86psM2jdfE+WVBIhJJJrcwPSHN'
    b'KoNpkhr5nfJEBt7aix6NoIZ+QvvutHRLx3WkPAmS1tkgziHadc9XGtY0VCuo8UdU3jlIsCot'
    b'r7nL43lR7Do78SsBNRlguSZXy0NcKYjqZaucobKAmp5i8rOTlPsHUNEWWJirz0nbLNleGkN7'
    b'dw+VqiQqASZcsk0a6LlS7a/aZAHNtnFe3YkfaJjDV4kf2IT+9+U6a4jFJZN15EpCAAduejyS'
    b'03I4BfYYdA1vbLHUblThx3OqcvyQUrtQahvK2Ynyfz2j3KPvqX9fQA0mUOcMLBPOi/PHgtUh'
    b'E82z7Xzy9ioGtqaIeFK6L/SSfuEJliTt/KKQUs5nbbwTLj/ch8fid0nwMGRi8Icwq7XGbFBl'
    b't3CzNex0SWqdEbbkBZaknV/C+srjF3EO9TAkGBFUm1qQ6ZJeA298YPJRj2JHKsI60RI5j8k7'
    b'AaPXpXxJYlLsbgvusRwoeCamhi4SDYa9IlgvSDbuwoYJWRA21Sew2BTT8wGae+kR8QbMxrnb'
    b'UGClsX4m5/8B735J6fL0cCgAAAAASUVORK5CYII=')

